"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ModelNode = (function () {
    function ModelNode(entity, id, connections) {
        this._entity = entity;
        this._id = id;
        this._connections = connections || [];
        this._sum = (connections) ? this._getSummedOcurrences() : 0;
    }
    Object.defineProperty(ModelNode.prototype, "value", {
        get: function () {
            return this._entity;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ModelNode.prototype, "id", {
        get: function () {
            return this._id;
        },
        enumerable: true,
        configurable: true
    });
    ModelNode.prototype.connect = function (toNode) {
        var id = toNode.id;
        var index = -1;
        this._connections.forEach(function (node, idx) {
            if (node.id === id) {
                index = idx;
            }
        });
        if (index === -1) {
            this._connections.push({
                id: id,
                occurences: 1
            });
        }
        else {
            this._connections[index].occurences++;
        }
        this._sum++;
    };
    ModelNode.prototype.connectedTo = function () {
        return this._connections;
    };
    ModelNode.prototype.probabilityOf = function (id) {
        for (var _i = 0, _a = this._connections; _i < _a.length; _i++) {
            var connection = _a[_i];
            if (connection.id === id) {
                return connection.occurences / this._sum;
            }
        }
        return 0;
    };
    ModelNode.prototype._getSummedOcurrences = function () {
        return this._connections.reduce(function (ret, ele) { return (ret + ele.occurences); }, 0);
    };
    return ModelNode;
}());
exports.ModelNode = ModelNode;
var MarkovModel = (function () {
    function MarkovModel() {
        this._nodes = [];
        this._keys = {};
    }
    MarkovModel.prototype.append = function (entities) {
        var _this = this;
        entities.reduce(function (previousNode, newEntity) {
            var newKey = newEntity.toString();
            var id;
            var thisNode;
            if (!_this._keys[newKey]) {
                id = _this._nodes.length;
                thisNode = new ModelNode(newEntity, id);
                _this._keys[newKey] = _this._nodes.push(thisNode) - 1;
            }
            else {
                thisNode = _this._nodes[_this._keys[newKey]];
            }
            if (previousNode) {
                previousNode.connect(thisNode);
            }
            return thisNode;
        }, undefined);
    };
    MarkovModel.prototype.getProbability = function (entities) {
        var _this = this;
        var ret = [];
        entities
            .map(function (entity) { return (_this._keys[entity.toString()]); })
            .reduce(function (prevE, thisE) {
            var res = (_this._nodes[prevE]) ? _this._nodes[prevE].probabilityOf(thisE) : 0;
            ret.push(res);
            return thisE;
        });
        return ret;
    };
    MarkovModel.prototype.walk = function (start) {
        var activeConnection;
        var activeNode;
        var activeNodeId;
        var nextNodeId;
        var anotherNumber;
        var result = [];
        if (this._keys[start.toString()]) {
            activeNodeId = this._keys[start.toString()];
            console.log(this._keys[start.toString()]);
            while (true) {
                activeNode = this._nodes[activeNodeId];
                activeConnection = activeNode.connectedTo();
                anotherNumber = this._dice(activeConnection);
                if (!activeConnection[anotherNumber] || activeConnection.length < 0) {
                    break;
                }
                nextNodeId = activeConnection[anotherNumber].id;
                result.push(this._nodes[activeNodeId].value);
                activeNodeId = nextNodeId;
            }
        }
        return result;
    };
    MarkovModel.prototype.dump = function () {
        return JSON.stringify(this);
    };
    MarkovModel.prototype._dice = function (connections) {
        var random = Math.random();
        var probs = connections.map(function (e) { return (e.occurences); });
        var max = probs.reduce(function (a, e) { return (a + e); }, 0);
        var index = 0;
        probs
            .sort(function (a, b) { return (b - a); })
            .map(function (e) { return (e / max); })
            .reduce(function (a, e, i) {
            if (a > random && a + e >= random) {
                index = i;
            }
            return a + e;
        }, 0);
        return index;
    };
    return MarkovModel;
}());
exports.MarkovModel = MarkovModel;
//# sourceMappingURL=MarkovModel.js.map