"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var StringAnalyzer = (function () {
    function StringAnalyzer() {
    }
    StringAnalyzer.prototype.reduce = function (input) {
        return input.reduce(function (ret, ele) {
            ret.push.apply(ret, ele);
            return ret;
        });
    };
    return StringAnalyzer;
}());
exports.StringAnalyzer = StringAnalyzer;
var NGramAnalyzer = (function (_super) {
    __extends(NGramAnalyzer, _super);
    function NGramAnalyzer(length) {
        var _this = _super.call(this) || this;
        _this._length = Math.floor(length);
        return _this;
    }
    NGramAnalyzer.prototype.analyze = function (string) {
        return this.reduce(string.map(this._split, this));
    };
    NGramAnalyzer.prototype._split = function (string) {
        var ret = [];
        ret[0] = string[0] + '-';
        if (string.length < this._length) {
            ret.push(string);
        }
        else {
            for (var i = 0; i < string.length - (this._length - 1); i++) {
                ret.push(string.substr(i, this._length));
            }
        }
        ret.push('-' + string[string.length - 1]);
        return ret;
    };
    return NGramAnalyzer;
}(StringAnalyzer));
exports.NGramAnalyzer = NGramAnalyzer;
var WhitespaceAnalyzer = (function (_super) {
    __extends(WhitespaceAnalyzer, _super);
    function WhitespaceAnalyzer() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    WhitespaceAnalyzer.prototype.analyze = function (string) {
        return this.reduce(string.map(function (element) { return (element.split(' ')); }));
    };
    return WhitespaceAnalyzer;
}(StringAnalyzer));
exports.WhitespaceAnalyzer = WhitespaceAnalyzer;
var LowercaseAnalyzer = (function (_super) {
    __extends(LowercaseAnalyzer, _super);
    function LowercaseAnalyzer() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    LowercaseAnalyzer.prototype.analyze = function (string) {
        return string.map(function (e) { return e.toLowerCase(); });
    };
    return LowercaseAnalyzer;
}(StringAnalyzer));
exports.LowercaseAnalyzer = LowercaseAnalyzer;
var AnalyzedString = (function () {
    function AnalyzedString(input) {
        if (typeof input === 'string') {
            this._store = [input];
        }
        else {
            this._store = input;
        }
    }
    AnalyzedString.prototype.whitespace = function () {
        return new AnalyzedString(this._reduce(this._store.map(function (e) { return (e.split(' ')); })));
    };
    AnalyzedString.prototype.ngram = function (length) {
        var _this = this;
        return new AnalyzedString(this._reduce(this._store.map(function (e) {
            return _this._split(e, length);
        }, this)));
    };
    AnalyzedString.prototype.lowercase = function () {
        var ret = this._store.map(function (e) { return (e.toLowerCase()); });
        return new AnalyzedString(ret);
    };
    AnalyzedString.prototype.contents = function () {
        return this._store;
    };
    AnalyzedString.prototype.compare = function (mode, other) {
        switch (mode) {
            case 'ngram3':
                var thisParts = this.lowercase().ngram(3).contents();
                var otherParts_1 = other.lowercase().ngram(3).contents();
                var intersectedParts = thisParts.filter(function (e) { return (otherParts_1.indexOf(e) !== -1); }).length;
                return 2 * intersectedParts / (thisParts.length + otherParts_1.length);
            default:
                return -1;
        }
    };
    AnalyzedString.prototype._split = function (string, length) {
        var ret = [];
        ret[0] = string[0] + '-';
        if (string.length < length) {
            ret.push(string);
        }
        else {
            for (var i = 0; i < string.length - (length - 1); i++) {
                ret.push(string.substr(i, length));
            }
        }
        ret.push('-' + string[string.length - 1]);
        return ret;
    };
    AnalyzedString.prototype._reduce = function (input) {
        return input.reduce(function (ret, ele) {
            ret.push.apply(ret, ele);
            return ret;
        });
    };
    return AnalyzedString;
}());
exports.AnalyzedString = AnalyzedString;
//# sourceMappingURL=StringAnalyzer.js.map